<?php

/**
 * @file
 * Form builder, submit, validate and operation functions for managing
 * complaints about comments.
 */

/**
 * Menu callback; administration of listing complaints on comments.
 */
function comment_abuse_admin() {
  $edit = $_POST;
  if (isset($edit['operation']) && ($edit['operation'] == 'delete') && isset($edit['comments']) && $edit['comments']) {
    module_load_include('inc', 'comment', 'comment.admin');
    return drupal_get_form('comment_abuse_multiple_delete_confirm');
  }
  else {
    return drupal_get_form('comment_abuse_list_form');
  }
}

/**
 * Form builder. Builds the complaints on comments overview form for the admin.
 */
function comment_abuse_list_form() {
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#attributes' => array('class' => array('container-inline')),
  );

  $options = array();
  $options['publish'] = t('Publish the selected comments');
  $options['delete'] = t('Delete the selected comments');
  $options['unpublish'] = t('Unpublish the selected comments');
  $options['remove_complaint'] = t('Remove all complaints for selected comments');

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#title_display' => 'invisible',
    '#options' => $options,
    '#default_value' => 'publish',
  );

  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );

  // Limit complaints on comments on page.
  global $base_url;
  $url = $base_url . '?';
  foreach ($_REQUEST as $key => $value) {
    $url .= ($key != 'q' && $key != 'page-limit') ? '&' : '';
    // Don't add old page-limit parameter.
    $url .= ($key != 'page-limit') ? "$key=$value" : '';
  }

  $output = t('<b>Limit complaints on page: </b>');
  $output .= ' ' . l(10, $url . '&page-limit=10');
  $output .= ' ' . l(25, $url . '&page-limit=25');
  $output .= ' ' . l(50, $url . '&page-limit=50');
  $output .= ' ' . l(100, $url . '&page-limit=100');
  $output .= ' ' . l(t('Show all'), $url . '&page-limit=ALL');

  $form['options']['markup'] = array(
    '#markup' => $output,
  );

  // Load the comments that need to be displayed.
  $header = array(
    'cid' => array('data' => t('Comment ID'), 'field' => 'ca.cid'),
    'status' => array('data' => t('Status'), 'field' => 'c.status'),
    'complaints' => array('data' => t('Complaints'), 'field' => 'complaint_count'),
    'subject' => array('data' => t('Subject'), 'field' => 'subject'),
    'author' => array('data' => t('Author'), 'field' => 'name'),
    'posted_in' => array('data' => t('Posted in'), 'field' => 'node_title'),
    'time' => array('data' => t('Time'), 'field' => 'ca.timestamp', 'sort' => 'desc'),
    'operations' => array('data' => t('Operations')),
  );

  // Limiting results on page.
  if (isset ($_GET['page-limit']) && ((int) ($_GET['page-limit']) != 0 || $_GET['page-limit'] == 'ALL')) {
    $page_limit = $_GET['page-limit'];
  }
  else {
    $page_limit = __COMMENT_ABUSE_DEFAULT_COMPLAINTS_ON_PAGE;
  }

  $query = db_select('comment', 'c')->extend('TableSort');
  $query->join('node', 'n', 'n.nid = c.nid');
  $query->join('comment_abuse', 'ca', 'c.cid = ca.cid');
  $query->addField('n', 'title', 'node_title');
  $query->addExpression('COUNT(ca.cid)', 'complaint_count');
  $query->addTag('node_access');
  if ($page_limit != 'ALL') {
    $query->range(0, $page_limit);
  }
  $query->groupBy('ca.cid');
  $result = $query
    ->fields('c', array('cid', 'subject', 'name', 'status'))
    ->fields('ca', array('timestamp'))
    ->orderByHeader($header)
    ->execute();

  // We collect a sorted list of node_titles during the query to attach to the
  // comments later.
  $complaint_comment = array();
  $cids = array();
  foreach ($result as $row) {
    $complaint_comment[$row->cid] = $row;
    $cids[] = $row->cid;
    $node_titles[] = $row->node_title;
  }

  $comments = comment_load_multiple($cids);

  // Build a table listing the appropriate comments.
  $options = array();
  $destination = drupal_get_destination();

  foreach ($comments as $comment) {
    // Remove the first node title from the node_titles array and attach to
    // the comment.
    $comment->node_title = array_shift($node_titles);
    $options[$comment->cid] = array(
      'cid' => $complaint_comment[$comment->cid]->cid,
      'status' => $complaint_comment[$comment->cid]->status,
      'complaints' => $complaint_comment[$comment->cid]->complaint_count,
      'subject' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $comment->subject,
          '#href' => 'comment/' . $comment->cid,
          '#options' => array('attributes' => array('title' => truncate_utf8($comment->comment_body[LANGUAGE_NONE][0]['value'], 128)), 'fragment' => 'comment-' . $comment->cid),
        ),
      ),
      'author' => theme('username', array('account' => $comment)),
      'posted_in' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $comment->node_title,
          '#href' => 'node/' . $comment->nid,
        ),
      ),
      'time' => format_date($complaint_comment[$comment->cid]->timestamp, 'medium'),
      'operations' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => t('edit'),
          '#href' => 'comment/' . $comment->cid . '/edit',
          '#options' => array('query' => $destination),
        ),
      ),
    );
  }

  $form['comments'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No complaints about the comments.'),
  );

  return $form;
}

/**
 * Validate comment_abuse_admin_overview form submissions.
 *
 * We can't execute any 'Update options' if no comments were selected.
 */
function comment_abuse_list_form_validate($form, &$form_state) {
  $form_state['values']['comments'] = array_diff($form_state['values']['comments'], array(0));
  if (count($form_state['values']['comments']) == 0) {
    form_set_error('', t('Please select one or more comments to perform the update on.'));
  }
}

/**
 * Process admin form for complaints on comments list.
 */
function comment_abuse_list_form_submit(&$form, &$form_state) {
  $operation = $form_state['values']['operation'];
  $cids = $form_state['values']['comments'];
  // Remove complaints on comments.
  if ($operation == 'remove_complaint') {
    $cids = array_filter($cids);
    db_delete('comment_abuse')
    ->condition('cid', array_keys($cids), 'IN')
    ->execute();
  }
  else {
    if ($operation == 'delete') {
      comment_delete_multiple($cids);
    }
    else {
      foreach ($cids as $cid => $value) {
        $comment = comment_load($value);

        if ($operation == 'unpublish') {
          $comment->status = COMMENT_NOT_PUBLISHED;
        }
        elseif ($operation == 'publish') {
          $comment->status = COMMENT_PUBLISHED;
        }
        comment_save($comment);
      }
    }
  }

  drupal_set_message(t('The update has been performed.'));
  $form_state['redirect'] = __COMMENT_ABUSE_LISTING_PATH;
  cache_clear_all();
}

/**
 * List the selected comments and verify that the admin really wants to delete
 * them.
 * @see comment_abuse_multiple_delete_confirm_submit()
 */
function comment_abuse_multiple_delete_confirm($form, &$form_state) {
  $edit = $form_state['input'];

  $form['comments'] = array(
    '#prefix' => '<ul>',
    '#suffix' => '</ul>',
    '#tree' => TRUE,
  );
  // array_filter() returns only elements with actual values.
  $comment_counter = 0;
  foreach (array_filter($edit['comments']) as $cid => $value) {
    $comment = comment_load($cid);
    if (is_object($comment) && is_numeric($comment->cid)) {
      $subject = db_query('SELECT subject FROM {comment} WHERE cid = :cid', array(':cid' => $cid))->fetchField();
      $form['comments'][$cid] = array('#type' => 'hidden', '#value' => $cid, '#prefix' => '<li>', '#suffix' => check_plain($subject) . '</li>');
      $comment_counter++;
    }
  }
  $form['operation'] = array('#type' => 'hidden', '#value' => 'delete');

  if (!$comment_counter) {
    drupal_set_message(t('There do not appear to be any comments to delete, or your selected comment was deleted by another administrator.'));
    drupal_goto(__COMMENT_ABUSE_LISTING_PATH);
  }
  else {
    return confirm_form($form,
                        t('Are you sure you want to delete these comments and all their children?'),
                        __COMMENT_ABUSE_LISTING_PATH, t('This action cannot be undone.'),
                        t('Delete comments'), t('Cancel'));
  }
}

/**
 * Process comment_abuse_multiple_delete_confirm form submissions.
 *
 * Perform the actual comment deletion.
 */
function comment_abuse_multiple_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    comment_delete_multiple(array_keys($form_state['values']['comments']));
    cache_clear_all();
    $count = count($form_state['values']['comments']);
    watchdog('content', 'Deleted @count comments.', array('@count' => $count));
    drupal_set_message(format_plural($count, 'Deleted 1 comment.', 'Deleted @count comments.'));
  }
  $form_state['redirect'] = __COMMENT_ABUSE_LISTING_PATH;
}

/**
 * Form builder for comment abuse options.
 */
function comment_abuse_options_form() {
  $form['comment_abuse_complaint_received_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Comment abuse complaint received message'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_received_message',
      __COMMENT_ABUSE_COMPLAINT_RECEIVED_MESSAGE
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  $form['comment_abuse_complaint_not_received_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Comment abuse complaint not received message'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_not_received_message',
      __COMMENT_ABUSE_COMPLAINT_NOT_RECEIVED_MESSAGE
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  $form['comment_abuse_complaint_link_text'] = array(
    '#type' => 'textfield',
    '#title' => t('Complaint link text'),
    '#default_value' => variable_get(
      'comment_abuse_complaint_link_text',
      __COMMENT_ABUSE_COMPLAINT_LINK_TEXT
    ),
    '#required' => TRUE,
    '#size' => 80,
  );

  return system_settings_form($form);
}
